import pandas as pd
from sklearn.preprocessing import StandardScaler

class FeatureScaler:
    '''FeatureScaler:
    this class is dessigned to make featur scaling very simple and begginer friendly.
    this class has 2 input arguments.
    FeatureScaler(df,checker=2)
    df: which is the dataset that you will be applying standard scaller
    checker is the threshold that your columns will be evluated at, by default this variable is set to 2 but you can change
    this depending on what you need'''
    def __init__(self, df, checker = 2):
        self.df = df
        self.scalerchecker = []
        self.scaler = StandardScaler()
        self.next_one = []
        self.checker = checker
    def Check(self):

        for i in self.df.columns:
            scalerdata = self.scaler.fit(self.df[i].array.reshape(-1, 1))
            # scalerchecker[i] = []
            checker = scalerdata.mean_
            for j in range(len(self.df[i])):
                if checker > 1:
                    if (self.df[i][j] > checker * self.checker):
                        self.scalerchecker.append(i)
        for i in self.scalerchecker:
            if (i in self.next_one):
                pass
            elif (i not in self.next_one):
                self.next_one.append(i)
        if (len(self.next_one)>0):
            scalerdata = self.scaler.fit_transform(self.df[self.next_one])
            self.df.drop(columns = self.next_one, inplace=True)
            newdf = pd.DataFrame(scalerdata, columns=self.next_one)
            for  i in newdf.columns:
                self.df[i] = newdf[i]
        return (self.df)


from pandas.plotting import scatter_matrix
import matplotlib.pyplot as plt
from PyQt5 import QtWidgets
import sys
# from IslanderDataPreprocessing import DataVisulization

class DataCorrelation:
    '''this module allows you to be able to view the correlation values of your dataset
    allowing you the ability to prevent simple errors
    DataCorrelation(df = pandas dataframe)
    df: is where you will input the dataset you would like to evaluate

    Correlationmatrix(): is the method you call uppon to view which columns have
    correlation relationships

    LookingAtCorr() is the method is where you will actually make the changes to your dataset
    this method returns a pandas dataframe

    Check(): this method will call uppon both LookingAtCorr, and Correlationmatrix for you
    this method also will return a pandas dataframe.
    '''
    def __init__(self,df):
        self.df = df
        self.copy = self.ByeByeText(self.df)
        self.high_corr = {}
        self.corr = []

    def ByeByeText(self,df):
        copy = df.copy()

        for  i in copy.columns:
            if (copy[i].dtype == "object"):
                copy.drop(columns = i, inplace=True)
        return (copy)
    def FindingScreenSize_(self):
        app = QtWidgets.QApplication(sys.argv)
        screen = app.primaryScreen()
        size = screen.size()
        screensize = (size.width()/95-2, size.height()/96-2)
        return screensize
    def Correlationmatrix(self):
        '''Correlationmatrix(): is the method you call uppon to view which columns have
        correlation relationships'''
        for  i in self.copy.columns:
            for  j in self.copy.columns:
                if i == j:
                    pass
                else:
                    # print(j)
                    corr = self.copy[i].corr(self.copy[j])
                    if (corr>=0.5 or corr<=-0.5):
                        # print(corr)
                        if (i not in self.high_corr.keys()):
                            self.high_corr[i] = []
                        self.high_corr[i].extend([j,corr])
                        self.corr.append(corr)
        print("these are correlation values for each column")
        count = 0
        for i in self.high_corr.keys():
            print(f"{count}:{i},{self.high_corr[i]}")
            count += 1
    def LookingAtCorr(self):
        '''LookingAtCorr() is the method where you will actually make the changes to your dataset
            this method returns a pandas dataframe '''
        print("with the values you see up above do you with to see a a scatter matrix of them")
        choice = input("enter yes if you do")
        if choice.upper() == "YES":

            column = []
            matrix = []
            for i in self.copy.columns:
                column.append(i)
            print(column)
            while (choice.upper()!="Q"):
                count = 0
                for i in column:
                    print(f"{count}:{i}")
                    count+=1
                try:
                    index = int(input("enter the corresponding number of each column that you would like to see"))
                except:
                    print("seems you picked an option that was not available")
                matrix.append(column[index])
                choice = input("enter q to view the scatter matrix")
                column.pop(index)
            scatter_matrix(self.copy[matrix],figsize=self.FindingScreenSize_())
            plt.show()
        # choice = input("enter yes if there is a plot you would like to view in more depth")
        # if (choice.upper()=="YES"):
        #     for i in self.copy.columns:
        #         print(i)
        #     x = input("enter the name of the column you would like to be on the x axis")
        #     y = input("enter the name of the column you would like to be on the y axis")
        #     single_plot =DataVisulization(data = self.copy, type_of_plot="scatter",column_values_for_x=x,column_values_for_y=y)
        #     single_plot.driver()
    def combine(self):
        copy_of_copy = self.copy.copy()
        drop = []
        choice = input("enter yes to combine some of the columns")
        if (choice.upper() == "YES"):
            column = []
            for i in self.copy.columns:
                column.append(i)
            while(choice.upper()!="Q"):
                count = 0
                for  i in column:
                    print(f"{count}:{i}")
                    count+=1
                while (True):
                    try:
                        numerator = int(input("enter the number of the corresponding column you would like to be the numerator to be"))
                        if (numerator<= len(column)):

                            break
                        else:
                            print("please enter of of the numbers you see on the screen")
                    except ValueError:
                        print("please enter a number")
                while (True):
                    try:
                        denominator = int(input("enter the number of the corresponding column you would like to be the denominator to be"))
                        if (denominator<= len(column)):

                            break
                        else:
                            print("please enter of of the numbers you see on the screen")
                    except ValueError:
                        print("please enter a number")
                name_of_new_column = input("enter what you would like the new name of the column to be")
                self.copy[name_of_new_column]= self.copy[column[numerator]]/self.copy[column[denominator]]
                drop.append(column[numerator])
                drop.append(column[denominator])
                choice = input("enter q if that is all the columns you would like to combine")
            self.copy.drop(columns= drop, inplace=True)
            # print(self.copy.columns)
            choice = input("enter yes if you would like to view the new correlation matrix scores")
            if (choice.upper()=="YES"):
                self.high_corr.clear()
                self.corr.clear()
                self.Correlationmatrix()
                print("what do you think of those scores?")
                choice = input("enter yes if you would like to keep these new scores"
                               " or enter nothing to revert them back to the original")
                if (choice.upper()!="YES"):
                    self.copy = copy_of_copy
                elif(choice.upper()=="YES"):
                    self.df = self.copy
            return self.df
    def Check(self):
        '''
            Check(): this method will call uppon both LookingAtCorr, and Correlationmatrix for you
            this method also will return a pandas dataframe'''
        self.Correlationmatrix()
        self.LookingAtCorr()
        return self.df



import pandas as pd

class NotPandasDataFrame(Exception):
    pass
class DataDiscovery:
    def __init__(self,df):
        '''this class is designed to allow you the ability to evaluate your data
        so that you may get an idea of what you need to change in the dataset
        the best way to use this class is by actaully creating an instance of this
        class where it will automate everything
        DataDiscovery(df)
        df will be any pandas dataframe you wish to evaluate'''
        if (isinstance(df,pd.DataFrame)):
            self.df = df
            self.Head()
            self.Info()
            self.Describe()
            self.IsNull()
            print("that is all of the information I have for you today please have an awesome day")
        else:
            raise NotPandasDataFrame(df)
    def Head(self):
        choice = input("enter yes to view the output provided by the head method")
        if (choice.upper()=="YES"):
            print("the defualt amount of rows you will see is 5")
            try:
                values = int(input("press enter if that is fine or enter the amount you want you want to view"))
            except ValueError:
                values = None
            if(values is not None):
                print(self.df.head(values))
            else:
                print(self.df.head())
    def Info(self):
        choice = input("enter yes to view the output provided by the info method")
        if (choice.upper()=="YES"):
            print(self.df.info())
            objectlist = []
            for i in self.df.columns:
                if self.df[i].dtypes == "object":
                    objectlist.append(i)
            if(len(objectlist)):
                print(f"the columns called\n{objectlist}\n are object types")
                values = input("enter yes to view the values of these columns")
                count = 0
                while(values.upper()=="YES"):
                    print(self.df[objectlist[count]].value_counts())
                    count+=1
                    if count>=len(objectlist):
                        break
                    values = input("enter yes to view the next columns data")
    def Describe(self):
        print("the next table you will see is from the describe method")
        choice = input("enter yes to continue")
        if(choice.upper()=="YES"):
            print(self.df.describe())
    def IsNull(self):
        choice = input("enter yes to see if your dataset has any null values")
        if(choice.upper()=="YES"):
            isnull=[]
            for i in self.df.columns:
                if (self.df[i].isnull().values.any()):
                    isnull.append(i)
            if (isnull!=[]):
                print(f"your data set has {len(isnull)} columns with null values, would you like to see the column names\n"
                      "as well as the amount of null values in that column")
                columns = input("enter yes if you do")
                if columns.upper() == "YES":
                    for i in isnull:
                        print(f"In the column {i}, there are {self.df[i].isnull().sum().sum()} cells with null values")



import pandas as pd

class NeedsToBeADict(Exception):
    pass
class NeedsToBeAPandasDataFrame(Exception):
    pass
class InconsistentData:
    def __init__(self, df):
        '''this class is dessigned to help you in the process of correcting inconsitent data
        you have the ability to use use,
        seperatingwords(origin,change):
        this method is created so that you will be able to make sure all the columns
        names with more then one word is seperated correctly
        origin is the original format used to seperate the words
        change is the format you would like to be used to seperate words
        changeing_column_cases(case = "title")
        this method is used to correct the columns nanes so that they are all in
        full caps, full lower, or title case
        case will be used to tell the method what case you would like
        by defalut case will be set equal to title but by saying
        case = upper the column names will be put to full lower
        and the same for case = upper
        column_names_white_space():
        this method will be used to correct white space in column names
        data_white_space():
        this method will be used to correct white space in the dataset
        correcting(column_name, corrections ):
        this method is dessigned to help you make the needed changes to the data in the cells
        so that your data is more consistent
        column_name is the var used to identify which column will get the corrections
        corrections is the dictionary with the corrected valuescheck(seperatingwords = False, origin = "", corrections = "" , change_case = False,case = "title", correcting = False, column_name = "", cell_corrections=None):
        this methode is designed to automate all the steps. needed except you will have to provide some
        input arguments
        first is seperatingwords by defalut is false when you set this to true you will be calling the
        seperatingwords words method
        therefore you will have to add what the origin is set equal
        as well as corrections these will both be some kind string values
        next input value will be case

        change_case = False to be able to have all your column names changed to the same case you will want change the value of change_case to true
        case = "title"
        you can change this depending on how you would like to formate your column names
        when you want to correct specifica values in the data you will set correcting to true as well as
        column_name = to the column name that will get these corrections done
        then
        cell_corrections = to a dictionary
        the corrected pandas data frame will be return
        autocheck():
        does the same as what check does but walks you through the proccess of making all the changes
        resources():
        a method dessigned to give you links for more information on the class'''

        if (isinstance(df, pd.core.frame.DataFrame)):
            self.df = df
            self.copy_df = df.copy
        else:
            raise NeedsToBeAPandasDataFrame(self.df)

    def seperatingwords(self,origin, change):
        '''this method is created so that you will be able to make sure all the columns
        names with more then one word is seperated correctly
        origin is the original format used to seperate the words
        change is the format you would like to be used to seperate words'''
        for i in self.df.columns:
            if origin in i:
                self.df.rename(columns = {i:i.replace(origin,change)}, inplace = True )
        return self.df
    def changeing_column_cases(self, case = "title"):
        '''this method is used to correct the columns nanes so that they are all in
        full caps, full lower, or title case
        case will be used to tell the method what case you would like
        by defalut case will be set equal to title but by saying
        case = upper the column names will be put to full lower
        and the same for case = upper'''
        for i in self.df.columns:
            if (case.lower() == "title"):
                self.df.rename(columns = {i:i.title()}, inplace = True )
            if (case.lower() == "upper"):
                self.df.rename(columns = {i:i.upper()}, inplace = True )
            if (case.lower() == "lower"):
                self.df.rename(columns = {i:i.lower()}, inplace = True )
        return self.df
    def column_names_white_space(self):
        '''this method will be used to correct white space in column names'''

        for i in self.df.columns:
            self.df.rename(columns = {i:i.strip()}, inplace = True )
        return self.df
    def data_white_space(self):
        '''this method will be used to correct white space in the dataset'''

        for i in self.df.columns:
            if self.df[i].dtype == "O":
                for j in self.df[i]:
                    try:
                        self.df[i].replace(to_replace={j:j.strip()}, inplace=True)
                    except AttributeError:
                        pass
        return self.df
    def correcting(self, column_name, corrections ):
        '''this method is dessigned to help you make the needed changes to the data in the cells
        so that your data is more consistent
        column_name is the var used to identify which column will get the corrections
        corrections is the dictionary with the corrected values'''

        if isinstance(corrections,dict):
            self.df.replace(to_replace=corrections, inplace=True)
        else:
            raise NeedsToBeADict
        return self.df
    def check(self, seperatingwords = False, origin = "", corrections = "" , change_case = False,case = "title", correcting = False, column_name = "", cell_corrections=None):
        '''this methode is designed to automate all the steps. needed except you will have to provide some
        input arguments
        first is seperatingwords by defalut is false when you set this to true you will be calling the
        seperatingwords words method
        therefore you will have to add what the origin is set equal
        as well as corrections these will both be some kind string values
        next input value will be case

        change_case = False to be able to have all your column names changed to the same case you will want change the value of change_case to true
        case = "title"
        you can change this depending on how you would like to formate your column names
        when you want to correct specifica values in the data you will set correcting to true as well as
        column_name = to the column name that will get these corrections done
        then
        cell_corrections = to a dictionary
        the corrected pandas data frame will be return'''
        if seperatingwords:
            self.seperatingwords(origin = origin,change = corrections)
        if change_case:
            self.changeing_column_cases(case)
        self.column_names_white_space()
        self.data_white_space()
        if correcting:
            self.correcting(column_name = column_name, corrections = cell_corrections)
        return self.df
    def autocheck(self):
        '''this method with automate every step of this class'''
        for i in self.df.columns:
            print(i)
        print("these are the names of all your columns")
        user = input("enter yes if you would like to correct the ways the words are seperated")
        if user.upper() == "YES":
            origin = input("enter the original format of the names you would like to change")
            corrections = input("how would you like the format to be")
            self.seperatingwords(origin = origin, change =corrections)
        user = input("enter yes if you would like to have all the column names the same")
        if user.upper() == "YES":
            case = input("enter upper to have all the column names to be full upper case\n"
                         "enter lower to have all the column names to be full lower case\n"
                         "enter title to have all the column names to be in title case")
            self.changeing_column_cases(case = case)
        self.column_names_white_space()
        print("after all the corrections these are your title names")
        for i in self.df.columns:
            print(i)
        self.data_white_space()
        for i in self.df.columns:
            if self.df[i].dtype == "object":
                print(f"the {i} column is an object type")
                user = input("enter yes if you would like to to view what values are in this column")
                if user.upper() == "YES":
                    print(self.df[i].value_counts())

                    user = input("enter yes if there are some values you wish to correct")
                    if user.upper() == "YES":
                        values = [j for j in self.df[i].value_counts().items()]
                        corrections = {}

                        while user.lower() =="yes" or len(values)!=0:
                            count = 0
                            for i in values:
                                print(f"{count}:{i}")
                                count+=1

                            print("from the list above pick the number corresponding with the value you want to correcct or enter any key to quit")
                            try:
                                origin = int(input(":"))
                                new_choice = input("what would you like this value to be instead")
                                corrections[values[origin]]= new_choice
                                values.pop(origin)
                            except ValueError:
                                break

        return self.df


    def resources(self):
        git = "https://github.com/Islanderrobotics/inconsistent_data"
        youtube = "https://youtu.be/KycIq40BhhI"
        print(f"you can find more resurces on youtube at {youtube}")
        print(f"as well as on git hub at {git}")



from .encoder2 import Encoder
import pandas as pd
from sklearn.impute import SimpleImputer
class MissingValues:
    '''MissingValues is dessigned to make correcting missing values alot more accesable.
    MissingValues(df)
    df: is the inputted pandas dataframe what will have corrections made to it
    check is the method used to tell the module to start the corrections, this method will return the corrected dataframe
    if you wish to get the original dataframe call the copy variable.
    currently you are only able to use the median stratagy however other methods are in the work'''
    def __init__(self, df):
        self.copy = df
        data = Encoder(df = df)
        data.check()
        self.df = data.df

    def check(self):
        self.isnull = []
        for i in self.df.columns:
            if (self.df[i].isnull().values.any()):
                self.isnull.append(i)
        if (len(self.isnull) >0):
            self.Imputer()
        return self.df
    def Imputer(self):
        imputer = SimpleImputer(strategy="median")
        new_data = imputer.fit_transform(self.df)
        convert = pd.DataFrame(new_data, columns=self.df.columns,index=self.df.index)
        self.df = convert


from setuptools import setup, find_packages

VERSION = '2021.0.3'
DESCRIPTION = 'module designed to make your data preprocessing experience easier'
with open("README.md") as description:
    LONG_DESCRIPTION = description.read()

# Setting up
setup(
    name="irdatacleaning",
    version=VERSION,
    author="Islander Robotics (William McKeon)",
    author_email="<IslanderRobotics@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=["pandas","matplotlib","PyQt5","scikit-learn","IslanderDataPreprocessing"],
    keywords=['python',"Machine Learning", "Artificial Intelligence","Data Science","Data Cleaning"],
)



class Resources:
    def __init__(self):
        '''this class is used to allow you islanders the ability to get additional resources on the module or classes'''
        links = {}
        links["github"] = "https://github.com/Islanderrobotics/irdatacleaning"
        for i in links.keys():
            print(f"the link to {i} is {links[i]}")


if __name__ == '__main__':
    data = Resources()
